function [ AGnew, indexnew ] = findStrongComp( AG )
%        [ AGnew, indexnew ] = findStrongComp( AG )
%
%   transforms the adjacency matrix A of a directed graph
%   in block-triangular form;
%   then strongly connected vertices have neighbouring numbers
%     Anew - transformed adjacency matrix
%     indexnew - list of the new enumeration of the vertices 1, 2,..., N
%  
% J. Lunze
% 
% 25.01.2022
% Version v. 26.1.2022
%
%
[N,M]=size(AG);
%
% If A is not quadratic: Anew=NaNa
if N ~= M
   return
end
% The main diagonal of A must be zero
% to avoid numerical problems:
AG=AG-diag(diag(AG));
% The entries of A should be 1 or 0
AG=double(abs(AG)>0);
%  
%% 1. Determine the reachability matrix
%
R=eye(N,N);               % vertices are reachable by themselves
AGk=eye(N,N);              % variable for A^k
allreachable=0;
k1=0;
while allreachable==0 && k1 < N
    k1=k1+1;
    AGk=AGk*AG;              % determine A^k with k=k1
    Rnew=R+AGk;
    Rnew=double(abs(Rnew)>0);  % normalise the elements to {0,1}
    if sum(sum(Rnew-R))==0           % current iteration did not change R
       allreachable=1;               % hence, the complete reachability 
                                     % matrix has been obtained
    end
    R=Rnew;
end
% R - reachability matrix of the given graph
%
if R==ones(N,N) 
   return         % graph is strongly connected
end
%% 2. Decompose the set of vertices into strongly connected sets
%
%  Strongly connected components are indicated by the different columns 
%  of RO = R AND R'
RO=min(R, R');             % RO = R AND R' - symmetric component of R
% delete redundant line from RO to get T
T=RO(:,1);                 % use the first column of RO
q=1;                       % count the number of strongly connected components
for k1=2:N
    Test=RO(:,k1);         % take the column number k1
    if sum(ismember(T', Test', 'rows'))==0  % if this column is not a column of T
        q=q+1;      % ... a new connected component has been found
        T(:,q)=Test;      % ... and this column of RO is appended to T
    end
end
%  q - number of strongly connected components
%  T - every column of T includes the vertices of one strongly connected component
%      (N x q)-matrix
%
%% 3. Sort the strongly connected sets
%
RC=T'*R*T;                    %  (q x q)-reachability matrix of the condensed graph
RC=double(abs(RC)>0);
%  nT=sum(RC) is a row vector showing how many vertices of the condensed graph 
%  are reachable from each vertex
%
%  Sort the columns of T according to nT=sum(RC)
nT=sum(RC);
SortedColumns=sortrows([nT' T'],'descend');  
Ts=SortedColumns(:,2:end)';
%  Tsorted: columns indicate the strong components in required order
s=Ts*[1:q]';                     % vector includes the number of the sets 
                                 % which a vertex belongs to
SortedVertices=sortrows([s [1:N]'],'ascend');
indexnew=SortedVertices(:,2);    % set of indices in the order
                                 % for the transformed adjacency matrix
%
%% 4. Transform the adjacency matrix                                
AGnew=AG(indexnew,indexnew);         % equivalent to P'*A*P
end


